package scacchi.main;

import scacchi.main.EccezioniTesto.*;

/**
 * Classe contenitore per classi e metodi per la memorizzazione
 * e verifica di stringhe di testo per la rappresentazine di case della
 * scacchiera, mosse, posizioni di pezzo e liste dei suddetti elementi. <br>
 * Queste stringhe possono apparire negli argomenti dei comandi
 * inviati dall'interfaccia al modulo che implementa il gioco vero e 
 * proprio. <br>
 * La classe contiene solo elementi static, in particolare static nested
 * classes, e non pu&ograve; essere istanziata o estesa.
 * @author Stefano Guerrini
 * @version 04/12/10
 */
public final class ArgomentiComando {
    /** Questa classe non pu&ograve; essere istanziata. */
    private ArgomentiComando() {};

    /** Simboli dei pezzi. */
    static private char simboliPezzi[] = {'R', 'D', 'T', 'A', 'C', 'P'};

    static private final char RE = simboliPezzi[0];
    static private final char DONNA = simboliPezzi[1];
    static private final char TORRE = simboliPezzi[2];
    static private final char ALFIERE = simboliPezzi[3];
    static private final char CAVALLO = simboliPezzi[4];
    static private final char PEDONE = simboliPezzi[5];

    /** 
     * Verifica se un carattere &egrave; un simbolo di pezzo valido. <br>
     * Un carattere maiuscolo indica un pezzo BIANCO, un carattere minuscolo
     * un pezzo NERO.
     **/
    static private boolean invalidSimboloPezzo(char ch) {
	for (byte b = 0; b < simboliPezzi.length; b++)
	    if (Character.toUpperCase(simboliPezzi[b]) ==
		Character.toUpperCase(ch))
		return false;
	return true;
    }

    /** Verifica se un carattere &egrave; un simbolo di colonna valido */
    static private boolean invalidColonna(char ch) {
	int oc = ch - 'a';
	return (oc < 0 || oc > 7);
    }

    /** Verifica se un carattere &egrave; un simbolo di traversa valido */
    static private boolean invalidTraversa(char ch) {
	int ot = ch - '1'; 
	return (ot < 0 || ot > 7);
    }

    /**
     * Verifica se una stringa &egrave; una casa valida.
     * @return null nel caso di casa valida<br>
     *         l'eccezione corrispondente al problema verificatosi, altrimenti.
     */
    static NotazioneException invalidCasa(String s) {
	if (s.length() != 2)
	    return new CasaTestoException(s);
	if (invalidColonna(s.charAt(0)))
	    return new ColonnaTestoException(s, 0);
	if (invalidTraversa(s.charAt(1)))
	    return new TraversaTestoException(s, 1);
	return null;
    }

    /**
     * Verifica se una stringa &egrave; una mossa valida.
     * @return null nel caso di mossa valida<br>
     *         l'eccezione corrispondente al problema verificatosi, altrimenti.
     */
    static NotazioneException invalidMossa(String s) {
	int l = s.length();
	if (l < 4)
	    return new MossaTestoException(s);
	if (invalidColonna(s.charAt(0)))
	    return new ColonnaTestoException(s, 0);
	if (invalidTraversa(s.charAt(1)))
	    return new TraversaTestoException(s, 1);
	if (invalidColonna(s.charAt(2)))
	    return new ColonnaTestoException(s, 2);
	if (invalidTraversa(s.charAt(3)))
	    return new TraversaTestoException(s, 3);
	if (l > 4) {
	    if (l != 6 || s.charAt(4) != '=')
		return new MossaTestoException(s);
	    else if (invalidSimboloPezzo(s.charAt(5)) ||
		     (Character.toUpperCase(RE) ==
		      Character.toUpperCase(s.charAt(5))) ||
		     (Character.toUpperCase(PEDONE) ==
		      Character.toUpperCase(s.charAt(5))))
		return new SimboloPezzoException(s, 5);
	}
	return null;
    }

    /**
     * Verifica se una stringa &egrave; una posizione di pezzo valida.
     * @return null nel caso di posizione di pezzo valida<br>
     *         l'eccezione corrsipondente al problema verificatosi, altrimenti.
     */
    static NotazioneException invalidPosPezzo(String s) {
	if (s.length() != 3)
	    return new PosPezzoException(s);
	if (invalidSimboloPezzo(s.charAt(0)))
	    return new SimboloPezzoException(s, 0);
	if (invalidColonna(s.charAt(1)))
	    return new ColonnaTestoException(s, 1);
	if (invalidTraversa(s.charAt(2)))
	    return new TraversaTestoException(s, 2);
	return null;
    }
    
    /** 
     * Concatena un vettore di stringhe.
     * @param v    Il vettore di stringhe da concatenare.
     * @param sep  La stringa da usare come separatore nella concatnezione.
     **/
    static String join(String v[], String sep) {
	if (v.length == 0) return "";
	String s = v[0];
	for (byte b = 1; b < v.length; b++)
	    s = s + sep + v[b];
	return s;
    }

    /**
     * Classe astratta per la memorizzazione e verifica di una stringa. <br>
     * Al momento della creazione di una istanza della classe, il metodo
     * astratto {@link #invalidString} viene chiamato per verificare
     * la validit&agrave; della stringa.
     */
    static abstract class StringaTesto {
	/** La stringa memorizzata */
	private final String s;
	
	/** 
	 * Questo metodo viene invocato sulla stringa da memorizzare
	 * al momento della creazione di una istanza. <br>
	 * @return null   se la stringa non presenta problemi<br>
	 *         l'eccezione corrispondente al problema individuato,
	 *         se la stringa presenta problemi.
	 */
	abstract NotazioneException invalidString(String s);
	
	/**
	 * Memorizza la stringa dopo averla verificata per mezzo di
	 * {@link #invalidString}.
	 * @throws EccezioniNotazione.NotazioneException l'eccezione
	 *         corrispondente all'errore di notazione trovato
	 *         nella stringa.
	 */
	StringaTesto(String s) throws NotazioneException {
	    NotazioneException e = invalidString(s);
	    if (e != null) throw e;
	    this.s = s;
	}

	/** Restituisce la stringa memorizzata. */
	public String toString() { return this.s; }
    }

    /**
     * Classe per la memorizzazione e verifica di case della scacchiera.
     */
    static public class CasaTesto extends StringaTesto {
	/**
	 * Verifica la stringa e crea l'istanza richiamando il costruttore
	 * della super-classe.
	 */
	public CasaTesto(String s) throws NotazioneException { super(s); }
	
	/** Verifica che la stringa &egrave; una casa. */
	NotazioneException invalidString(String s) { return invalidCasa(s); }
    }

    /**
     * Classe per la memorizzazione e verifica di mosse.
     */
    static public class MossaTesto extends StringaTesto {
	/**
	 * Verifica la stringa e crea l'istanza richiamando il costruttore
	 * della super-classe.
	 */
	public MossaTesto(String s) throws NotazioneException { super(s); }

	/** Verifica che la stringa &egrave; una mossa. */
	NotazioneException invalidString(String s) { return invalidMossa(s); }
    }

    /**
     * Classe per la memorizzazione e verifica di posizioni di pezzo.
     */
    static public class PosPezzoTesto extends StringaTesto {
	/**
	 * Verifica la stringa e crea l'istanza richiamando il costruttore
	 * della super-classe.
	 */
	public PosPezzoTesto(String s) throws NotazioneException { super(s); }

	/** Verifica che la stringa &egrave; una posizione di pezzo. */
	NotazioneException invalidString(String s) {
	    return invalidPosPezzo(s); }
    }

    /**
     * Classe astratta per la memorizzazione di liste per la decomposizione
     * e analisi di una stringa nella sequenza di sottostringhe separate da
     * spazi bianchi che la compogono.
     */
    static abstract class ListaTesto {
	/** Memorizza la lista di stringhe. */
	private final String v[];

	/**
	 * Il metodo che verifica la validit&grave; di una stringa.
	 * @param s la stringa da verificare, &egrave; una delle
	 *          stringhe della lista.
	 */
	abstract NotazioneException invalidString(String s);

	/**
	 * Suddivide una stringa nella lista di sottostringhe
	 * separate da spazi che la compongono. <br>
	 * Richiama {@link #invalidString} su ogni sottostringa e, se
	 * il risultato &egrave; una eccezione diversa da null, lancia
	 * l'eccezione.
	 */
	ListaTesto(String s) throws NotazioneException {
	    s = s.trim();
	    if (s.length() == 0)
		this.v  = new String[0];
	    else {
		this.v = s.split("\\s+");

		for (byte b = 0; b < v.length; b++) {
		    NotazioneException e = invalidString(v[b]);
		    if (e != null) throw e;
		}
	    }
	}

	/** Ritorna il numero di sottostringhe. */
	public int length() { return v.length; }

	/** Ritorna il vettore delle sottostringhe. */
	public String[] v() { return v; }
	
	/** Ricostruisce la stringa, concatenando tutte le sottostringhe. */
	public String toString() { return join(this.v, " "); }
    }

    /**
     * Memorizza e decompone una stringa composta da case della scacchiera.
     */
    static public class ListaCaseTesto extends ListaTesto {
	/**
	 * Verifica la stringa e crea l'istanza richiamando il costruttore
	 * della super-classe.
	 */
	public ListaCaseTesto(String s) throws NotazioneException { super(s); }

	/** Verifica che ogni sottostringa &egrave; una casa. */
	NotazioneException invalidString(String s) { return invalidCasa(s); }
    }

    /**
     * Memorizza e decompone una stringa composta da mosse.
     */
    static public class ListaMosseTesto extends ListaTesto {
	/**
	 * Verifica la stringa e crea l'istanza richiamando il costruttore
	 * della super-classe.
	 */
	public ListaMosseTesto(String s) throws NotazioneException {super(s);}

	/** Verifica che ogni sottostringa &egrave; una mossa. */
	NotazioneException invalidString(String s) { return invalidMossa(s); }
    }

    /**
     * Memorizza e decompone una stringa composta da posizioni di pezzo.
     */
    static public class ListaPosPezziTesto extends ListaTesto {
	/**
	 * Verifica la stringa e crea l'istanza richiamando il costruttore
	 * della super-classe.
	 */
	public ListaPosPezziTesto(String s) throws NotazioneException {
	    super(s); }

	/** Verifica che ogni sottostringa &egrave; una posizione di pezzo. */
	NotazioneException invalidString(String s) {
	    return invalidPosPezzo(s); }
    }
}
